==> 5.0.0.tar.gz: test/riverpod_bloc_stream_test.dart <==
import 'package:bloc_stream/bloc_stream.dart';
import 'package:mockito/mockito.dart';
import 'package:riverpod/riverpod.dart';
import 'package:riverpod_bloc_stream/riverpod_bloc_stream.dart';
import 'package:test/test.dart';

import 'utils.dart';

void main() {
  test('BlocStreamFamily override', () {
    final provider =
        BlocStreamProvider.family<TestBloc, int>((ref, a) => TestBloc());
    final notifier2 = TestBloc(42);
    final container = ProviderContainer(
      overrides: [provider.overrideWithProvider((ref, a) => notifier2)],
    );

    // access in the child container
    // try to read provider.state before provider and see if it points to the override
    expect(container.read(provider(0).value), 42);
    expect(container.read(provider(0)), notifier2);
  });

  test('can be assigned to provider', () {
    final Provider<TestBloc> provider = BlocStreamProvider((_) {
      return TestBloc();
    });
    final container = ProviderContainer();

    expect(container.read(provider), isA<TestBloc>());
  });

  test('overriding the provider overrides provider.state too', () async {
    final notifier = TestBloc(42);
    final provider = BlocStreamProvider((_) => TestBloc());
    final container = ProviderContainer(
      overrides: [
        provider.overrideWithProvider(BlocStreamProvider((_) => TestBloc(10)))
      ],
    );

    // does not crash
    container.updateOverrides([
      provider.overrideWithProvider(BlocStreamProvider((_) => notifier)),
    ]);

    expect(container.read(provider), notifier);
    expect(container.read(provider.value), 42);

    notifier.add(increment);
    await Future.delayed(Duration(milliseconds: 0));

    expect(container.read(provider.value), 43);
  });

  test('can specify name', () {
    final provider = BlocStreamProvider(
      (_) => TestBloc(),
      name: 'example',
    );

    expect(provider.name, 'example');
    expect(provider.value.name, 'example.value');

    final provider2 = BlocStreamProvider((_) => TestBloc());

    expect(provider2.name, isNull);
    expect(provider2.value.name, isNull);
  });

  test('disposes the notifier when provider is unmounted', () {
    final bloc = TestBloc();
    final provider = BlocStreamProvider<TestBloc>((_) {
      return bloc;
    });
    final container = ProviderContainer();

    expect(container.read(provider), bloc);
    expect(bloc.closed, isFalse);

    container.dispose();

    expect(bloc.closed, isTrue);
  });

  test('provider subscribe the callback is never', () async {
    final bloc = TestBloc();
    final provider = BlocStreamProvider<TestBloc>((_) {
      return bloc;
    });
    final listener = ControllerListenerMock();
    final container = ProviderContainer();

    final sub = provider.addLazyListener(
      container,
      mayHaveChanged: () {},
      onChange: listener,
    );

    verify(listener(argThat(isA<TestBloc>()))).called(1);
    verifyNoMoreInteractions(listener);

    bloc.add(increment);

    verifyNoMoreInteractions(listener);
    sub.read();

    verifyNoMoreInteractions(listener);

    container.dispose();
    await Future.value(null);

    verifyNoMoreInteractions(listener);
  });

  test('provider subscribe callback never called', () async {
    final provider = BlocStreamProvider<TestBloc>((_) {
      return TestBloc();
    });
    final listener = ListenerMock();
    final container = ProviderContainer();

    final sub = provider.value.addLazyListener(
      container,
      mayHaveChanged: () {},
      onChange: listener,
    );

    verify(listener(0)).called(1);
    verifyNoMoreInteractions(listener);

    container.read(provider).add(increment);

    verifyNoMoreInteractions(listener);
    await Future.delayed(Duration(milliseconds: 0));
    sub.read();
    verify(listener(1)).called(1);
    verifyNoMoreInteractions(listener);

    container.dispose();
    await Future.value(null);

    verifyNoMoreInteractions(listener);
  });
}

void increment(TestBloc b, void Function(int) add) => add(b.value + 1);

class TestBloc extends BlocStream<int> {
  TestBloc([int initialValue = 0]) : super(initialValue);

  bool closed = false;

  @override
  Future<void> close() {
    closed = true;
    return super.close();
  }
}

class ListenerMock extends Mock {
  void call(int value);
}

class ControllerListenerMock extends Mock {
  void call(TestBloc? value);
}

==> 6.1.0.tar.gz: test/riverpod_bloc_stream_test.dart <==
import 'package:bloc_stream/bloc_stream.dart';
import 'package:mockito/mockito.dart';
import 'package:riverpod/riverpod.dart';
import 'package:riverpod_bloc_stream/riverpod_bloc_stream.dart';
import 'package:test/test.dart';

import 'utils.dart';

void main() {
  test('BlocStreamFamily override', () {
    final provider = BlocStreamProvider.family<TestBloc, int, Object?>(
        (ref, a) => TestBloc());
    final bloc2 = TestBloc(42);
    final container = ProviderContainer(
      overrides: [provider.overrideWithProvider((ref, a) => bloc2)],
    );

    // access in the child container
    // try to read provider.state before provider and see if it points to the override
    expect(container.read(provider(0)), 42);
    expect(container.read(provider(0).bloc), bloc2);
  });

  test('overriding the provider overrides provider.state too', () async {
    final bloc = TestBloc(42);
    final provider = BlocStreamProvider((_) => TestBloc());
    final container = ProviderContainer(
      overrides: [
        provider.overrideWithProvider(BlocStreamProvider((_) => TestBloc(10)))
      ],
    );

    // does not crash
    container.updateOverrides([
      provider.overrideWithProvider(BlocStreamProvider((_) => bloc)),
    ]);

    expect(container.read(provider), 42);
    expect(container.read(provider.bloc), bloc);

    bloc.add(increment);
    await Future.delayed(Duration(milliseconds: 0));

    expect(container.read(provider), 43);
  });

  test('can specify name', () {
    final provider = BlocStreamProvider(
      (_) => TestBloc(),
      name: 'example',
    );

    expect(provider.name, 'example');
    expect(provider.bloc.name, 'example.bloc');

    final provider2 = BlocStreamProvider((_) => TestBloc());

    expect(provider2.name, isNull);
    expect(provider2.bloc.name, isNull);
  });

  test('disposes the notifier when provider is unmounted', () {
    final bloc = TestBloc();
    final provider = BlocStreamProvider<TestBloc, int>((_) {
      return bloc;
    });
    final container = ProviderContainer();

    expect(container.read(provider.bloc), bloc);
    expect(bloc.closed, isFalse);

    container.dispose();

    expect(bloc.closed, isTrue);
  });

  test('provider subscribe the callback is never', () async {
    final bloc = TestBloc();
    final provider = BlocStreamProvider<TestBloc, int>((_) {
      return bloc;
    });
    final listener = ListenerMock();
    final container = ProviderContainer();

    final sub = provider.addLazyListener(
      container,
      mayHaveChanged: () {},
      onChange: listener,
    );

    verify(listener(0)).called(1);
    verifyNoMoreInteractions(listener);

    bloc.add(increment);

    verifyNoMoreInteractions(listener);
    sub.read();

    verifyNoMoreInteractions(listener);

    container.dispose();
    await Future.value(null);

    verifyNoMoreInteractions(listener);
  });

  test('provider subscribe callback never called', () async {
    final provider = BlocStreamProvider<TestBloc, int>((_) {
      return TestBloc();
    });
    final listener = ListenerMock();
    final container = ProviderContainer();

    final sub = provider.addLazyListener(
      container,
      mayHaveChanged: () {},
      onChange: listener,
    );

    verify(listener(0)).called(1);
    verifyNoMoreInteractions(listener);

    container.read(provider.bloc).add(increment);

    verifyNoMoreInteractions(listener);
    await Future.delayed(Duration(milliseconds: 0));
    sub.read();
    verify(listener(1)).called(1);
    verifyNoMoreInteractions(listener);

    container.dispose();
    await Future.value(null);

    verifyNoMoreInteractions(listener);
  });
}

void increment(TestBloc b, void Function(int) add) => add(b.value + 1);

class TestBloc extends BlocStream<int> {
  TestBloc([int initialValue = 0]) : super(initialValue);

  bool closed = false;

  @override
  Future<void> close() {
    closed = true;
    return super.close();
  }
}

class ListenerMock extends Mock {
  void call(int value);
}

class ControllerListenerMock extends Mock {
  void call(TestBloc? value);
}
